
-- ToME - Tales of Middle-Earth
-- Copyright (C) 2009 - 2014 Nicolas Casalini
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-- Nicolas Casalini "DarkGod"
-- darkgod@te4.org

-- Fears:
-- Paranoia: The same (chance to attack friend or foe). Level 2:
-- Terrified: % chance to fail attacks/talents
-- Haunted: fear immunity penetration + dam reduction (or old torment effect?)

-- Possibly new way of fear immunity working: x% fear immunity -> x% chance to ignore a fear/fear stack

--Ideas:
-- Talent 1: mostly the same
-- Talent 2: mostly same
-- Talent 3: focus your fear-ity on a single enemy, making them able to recieve more stacks on their fears
-- Talent 4: paralyses instead of fears away11

--TODO: paranoia doesnt scale with charges from heighten fear

local getTerrifiedActionFailureChance = function(self, t)
	return math.min(50, self:combatTalentMindDamage(t, 20, 45))
end

local getDistressedSaveChange = function(self, t)
	return -self:combatTalentMindDamage(t, 15, 30)
end

local getParanoidAttackChance = function(self, t)
	return math.min(60, self:combatTalentMindDamage(t, 30, 50))
end


local getHauntedDamageReduction = function(self, t)
	return self:combatTalentScale(t, 0.1, 0.3)
end

local getHauntedFearPenetration = function(self, t)
	return self:combatTalentScale(t, 0.2, 0.5)
end

local getTormentedCount = function(self, t)
	return 4 + math.min(5, math.floor(math.pow(self:getTalentLevelRaw(t), 0.7)))
end

local getTormentedDamage = function(self, t)
	return self:combatTalentMindDamage(t, 40, 60)
end

newTalent{
	name = "Instill Fear",
	type = {"cursed/fears", 1},
	require = cursed_wil_req1,
	points = 5,
	random_ego = "attack",
	cooldown = 6,
	hate = 8,
	range = 8,
	radius = function(self, t) return self:combatTalentScale(t, -1, 1.5, 0.5, 1) end,
	tactical = { DISABLE = 2 },
	getDuration = function(self, t)
		return 8
	end,
	getAoeApplyChance = function(self, t)
		return self:combatTalentScale(t, 40, 75)
	end,
	hasEffect = function(self, t, target)
		if not target then return false end
		if target:hasEffect(target.EFF_PARANOID) then return true end
		if target:hasEffect(target.EFF_DISPAIR) then return true end
		if target:hasEffect(target.EFF_TERRIFIED) then return true end
		if target:hasEffect(target.EFF_DISTRESSED) then return true end
		if target:hasEffect(target.EFF_HAUNTED) then return true end
		if target:hasEffect(target.EFF_TORMENTED) then return true end
		return false
	end,
	applyEffect = function(self, t, target)
		local tHeightenFear, tTyrant
		if self:knowTalent(self.T_HEIGHTEN_FEAR) then tHeightenFear = self:getTalentFromId(self.T_HEIGHTEN_FEAR) end
		if self:knowTalent(self.T_TYRANT) then tTyrant = self:getTalentFromId(self.T_TYRANT) end
		local mindpowerChange = tTyrant and tTyrant.getMindpowerChange(self, tTyrant) or 0

		local mindpower = self:combatMindpower(1, mindpowerChange)
		if not target:checkHit(mindpower, target:combatMentalResist()) then
			game.logSeen(target, "%s resists the fear!", target.name:capitalize())
			return nil
		end

		local effects = {}
		if target.fear_immune and target.fear_immune ~= 0 then --FORGER:Previously creatures without fear resist were getting Haunted effect, so we check if fear immunity isn't nil
			-- Currently we let fear immunity still matter by giving it a chance to deny fears (apart from the first fear penetrating haunting)
			if not target:hasEffect(target.EFF_HAUNTED) then
				table.insert(effects, target.EFF_HAUNTED)
			end
		end
		if target:canBe("fear") then --FORGER:We should do the rng check here instead, all of this accomplishes the same effect without giving Haunted effect to everything
			if not target:hasEffect(target.EFF_PARANOID) then table.insert(effects, target.EFF_PARANOID) end
			if not target:hasEffect(target.EFF_TERRIFIED) then table.insert(effects, target.EFF_TERRIFIED) end
			if not target:hasEffect(target.EFF_HAUNTED) then table.insert(effects, target.EFF_HAUNTED) end
		end

		if #effects == 0 then return nil end
		local effectId = rng.table(effects)

		local duration = t.getDuration(self, t)
		local eff = {src=self, duration=duration }
		if effectId == target.EFF_PARANOID then
			eff.attackChance = getParanoidAttackChance(self, t)
			eff.mindpower = mindpower
		elseif effectId == target.EFF_TERRIFIED then
			eff.actionFailureChance = getTerrifiedActionFailureChance(self, t)--FORGER:t was originally tHeightenFear, I didn't know if this was intended but now it works
		elseif effectId == target.EFF_HAUNTED then
			eff.damageReduction = -getHauntedDamageReduction(self, t)*100
			eff.fear_penetration = getHauntedFearPenetration(self, t)
		elseif effectId == target.EFF_TORMENTED then
			eff.count = getTormentedCount(self, tTyrant)
			eff.damage = getTormentedDamage(self, tTyrant)
			eff.counts = {}
			for i = 1, duration do
				eff.counts[i] = math.floor(eff.count / duration) + ((eff.count % duration >= i) and 1 or 0)
			end
		else
			print("* fears: failed to get effect", effectId)
		end

		target:setEffect(effectId, duration, eff)


		return effectId
	end,
	endEffect = function(self, t)
		local tHeightenFear
		if self:knowTalent(self.T_HEIGHTEN_FEAR) then tHeightenFear = self:getTalentFromId(self.T_HEIGHTEN_FEAR) end
		if tHeightenFear then
			if not t.hasEffect(self, t) then
				-- no more fears
				self:removeEffect(self.EFF_HEIGHTEN_FEAR)
			end
		end
	end,
	requires_target = true,
	target = function(self, t) return {type="ball", range=self:getTalentRange(t), radius=self:getTalentRadius(t), talent=t} end,
	action = function(self, t)
		local tg = self:getTalentTarget(t)
		local x, y, target = self:getTarget(tg)
		if not x or not y or not target or core.fov.distance(self.x, self.y, x, y) > self:getTalentRange(t) then return nil end

		self:project(
			tg, x, y,
			function(px, py)
				local actor = game.level.map(px, py, engine.Map.ACTOR)
				if actor and self:reactionToward(actor) < 0 and actor ~= self then
					if actor == target or rng.percent(t.getAoeApplyChance(self, t)) then
						local tInstillFear = self:getTalentFromId(self.T_INSTILL_FEAR)
						tInstillFear.applyEffect(self, tInstillFear, actor)
					end
				end
			end,
			nil, nil)

		return true
	end,
	info = function(self, t)
		-- return ([[Instill fear in your target, causing one of several possible fears that lasts for %d turns. There is also a 25%% chance of instilling fear in any foe in a radius of %d. The target can save versus Mindpower to resist the effect, and can be affected by multiple fears.
		-- You gain 2 new fears: The Paranoid effect gives the target an %d%% chance to physically attack a nearby creature, friend or foe. If hit, their target will be afflicted with Paranoia as well. The Despair effect reduces the target's resistance to all damage by %d%%.
		-- Fear effects improve with your Mindpower.]]):format(t.getDuration(self, t), self:getTalentRadius(t),
		-- t.getParanoidAttackChance(self, t),
		-- -t.getDespairResistAllChange(self, t))
		return ([[Instill fear in your target, causing one of several possible fears that lasts for %d turns. There is also a %d%% chance of instilling fear in any foe in a radius of %d. The target can save versus Mindpower to resist the effect, and can be affected by multiple fears. If the target has fear immunity, the haunted fear will be applied.

		Fear effects:
		Paranoia gives the target an %d%% chance to physically attack a nearby creature, friend or foe. If hit, their target will be afflicted with Paranoia as well.
		Haunted reduces the target's fear resistance by %d%% and damage dealt by %d%%.
		Terrified causes the target to fail their attacks and talents %d%% of the time.]]):format(
			t.getDuration(self, t),
			t.getAoeApplyChance(self, t),
			self:getTalentRadius(t),
			getParanoidAttackChance(self, t),
			getHauntedFearPenetration(self, t)*100,
			getHauntedDamageReduction(self, t)*100,
			getTerrifiedActionFailureChance(self, t))
	end,
}

newTalent{
	name = "Heighten Fear",
	type = {"cursed/fears", 2},
	require = cursed_wil_req2,
	mode = "passive",
	points = 5,
	radius = function(self, t)
		return math.sqrt(self:getTalentLevel(t)) * 3
	end,
	getTurnsUntilTrigger = function(self, t)
		return 5
	end,
	tactical = { DISABLE = 2 },
	getTerrifiedResistAllChange = function(self, t)
		return -self:combatTalentMindDamage(t, 15, 40)
	end,
	callbackOnActBase = function(self, t)
		local tg = {type="ball", range=0, selffire=false, radius=self:getTalentRadius(t)}
		self:project(tg, self.x, self.y, function(px, py, tg, self)
			local target = game.level.map(px, py, Map.ACTOR)
			if target then
				local has_fear = target:hasEffect(target.EFF_PARANOID) or target:hasEffect(target.EFF_TERRIFIED) or target:hasEffect(target.EFF_HAUNTED)
				if has_fear then
					local turnsUntilTrigger = t.getTurnsUntilTrigger(self, t)
					target:setEffect(target.EFF_HEIGHTEN_FEAR, 5, {src=self, range=self:getTalentRadius(t), turns=turnsUntilTrigger, turns_left=turnsUntilTrigger })
				end
			end
		end)
	end,
	info = function(self, t)
		local tInstillFear = self:getTalentFromId(self.T_INSTILL_FEAR)
		local radius = self:getTalentRadius(t)
		local turnsUntilTrigger = t.getTurnsUntilTrigger(self, t)
		local duration = tInstillFear.getDuration(self, tInstillFear)
		return ([[Heighten the fears of everyone around you. Any foe you inflict at least one fear upon and who remains in a radius of %d and in sight of you for %d (non-consecutive) turns, will gain a new fear that lasts for %d turns. Also, every turn they are in range and in sight, their fears will increase in power (twice as strong after 5 turns). After 5 turns, the fears will apply a secondary effect.

		Secondary effects:
		Paranoia will do x
		Haunted will do x
		Terrified will now also reduce all resistances by %d%%.
		]]):format(radius, turnsUntilTrigger, duration,
		-t.getTerrifiedResistAllChange(self, t))
	end,
}

newTalent{
	name = "Tyrant",
	type = {"cursed/fears", 3},
	mode = "passive",
	require = cursed_wil_req3,
	points = 5,
	on_learn = function(self, t)
	end,
	on_unlearn = function(self, t)
	end,
	getMindpowerChange = function(self, t)
		return math.floor(math.sqrt(self:getTalentLevel(t)) * 7)
	end,
	info = function(self, t)
		return ([[Impose your tyranny on the minds of those who fear you. Your mindpower is increased by %d against foes who attempt to resist your fears.
		]]):format(t.getMindpowerChange(self, t))
	end,
}

newTalent{
	name = "Panic",
	type = {"cursed/fears", 4},
	require = cursed_wil_req4,
	points = 5,
	random_ego = "attack",
	cooldown = 20,
	hate =  1,
	range =  function(self, t)
		return math.floor(self:combatTalentScale(t, 3, 5))
	end,
	tactical = { DISABLE = 4 },
	getDuration = function(self, t)
		return 3 + math.floor(math.pow(self:getTalentLevel(t), 0.5) * 2.2)
	end,
	getChance = function(self, t)
		return self:combatTalentScale(t, 15, 30)
	end,
	action = function(self, t)
		local range = self:getTalentRange(t)
		local duration = t.getDuration(self, t)
		local chance = t.getChance(self, t)
		self:project(
			{type="ball", radius=range}, self.x, self.y,
			function(px, py)
				local actor = game.level.map(px, py, engine.Map.ACTOR)
				if actor and self:reactionToward(actor) < 0 and actor ~= self then
					if not actor:canBe("fear") then
						game.logSeen(actor, "#F53CBE#%s ignores the panic!", actor.name:capitalize())
					elseif actor:checkHit(self:combatMindpower(), actor:combatMentalResist(), 0, 95) then
						actor:setEffect(actor.EFF_PANICKED, duration, {src=self,range=10,chance=chance})
					else
						game.logSeen(actor, "#F53CBE#%s resists the panic!", actor.name:capitalize())
					end
				end
			end,
			nil, nil)
		return true
	end,
	info = function(self, t)
		local range = self:getTalentRange(t)
		local duration = t.getDuration(self, t)
		local chance = t.getChance(self, t)
		--return ([[Panic your enemies within a range of %d for %d turns. Anyone who fails to make a mental save against your Mindpower has a %d%% chance each turn of trying to run away from you.]]):format(range, duration, chance)
		return ([[Panic your enemies within a range of %d for %d turns. Anyone who fails to make a mental save against your Mindpower has a %d%% chance each turn of freezing in fear, losing their turn. Enemies that freeze in fear that have fears on them will also level up one of their level 1 fears.]]):format(range, duration, chance)
	end,
}
